"""
@File  : report.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  : 
"""
# -*- coding: utf-8 -*-
import datetime
import os
import re
import sys

from docx import Document
from docx.document import Document as DocObject
from docx.enum.section import WD_ORIENTATION
from docx.enum.text import WD_BREAK
from docx.oxml import OxmlElement, ns
from docx.shared import Cm

from smartDocx.config import PaperTitleStyleBase, PaperSubTitleStyleBase, AbstractStyleBase, \
    FirstTitleStyleBase, SecondTitleStyleBase, ThirdTitleStyleBase, FourthTitleStyleBase, ContentStyleBase, \
    TableTitleStyleBase, TableHeadStyleBase, TableBodyStyleBase, PicTitleStyleBase, PageStyleBase
from smartDocx.constants import WORD_BUILTIN_STYLES, TEMPLATE_DIR, Orientation


class SmartReportDataBase(object):
    """智能报告数据"""

    def __init__(self, sql_client=None, neo4j_client=None, es_client=None, *args, **kwargs):
        self._data = {
            'global': dict(
                sql_client=sql_client,
                neo4j_client=neo4j_client,
                es_client=es_client
            )
        }
        self.sql_client = sql_client
        self.neo4j_client = neo4j_client
        self.es_client = es_client
        self.args = args
        self.kwargs = kwargs

    @property
    def data(self):
        """获取报告数据"""
        self.distribute()
        return self._data

    def distribute(self):
        """派发任务"""
        for attr in dir(self):
            if attr.startswith('chapter_'):
                handler = getattr(self, attr)
                handler()

    def get_chapter_level(self, chapter_function_name: str):
        """获取章节层级"""
        return chapter_function_name.split('chapter_')[1]


class SmartReportBase(object):
    """智能报告"""

    def __init__(self, filename: str, data: dict, template: str = None, *args, **kwargs):
        """
        智能报告

        :param filename: 文件绝对路径
        :param data: 数据
        :param args:
        :param kwargs:
        """
        self.args = args
        self.kwargs = kwargs
        if not os.path.isabs(filename):
            filename = os.path.abspath(filename)
        self.filename = filename
        self.data = data
        if template:
            # 读取模板文件, 路径检索顺序: 模板文件夹>全局检索
            if not os.path.exists(os.path.join(TEMPLATE_DIR, template)):
                if not os.path.exists(template):
                    raise FileNotFoundError(template)
            else:
                template = os.path.join(TEMPLATE_DIR, template)
            print('从模板文件创建: {}'.format(template))
            self.doc_obj = Document(template)  # type: DocObject
        else:
            # 创建新文件
            self.doc_obj = Document()  # type: DocObject
            print('从新文件创建: {}'.format(template))
        self._clear_files = set()
        # 页面布局(cm):默认A4纸大小
        self.page_conf = PageStyleBase()
        self.default_orientation = self.page_conf['orientation']
        self._load_builtin_style()
        self._handle_filename()

        # 默认样式设置
        # 文章标题
        self.title_level_0_style = PaperTitleStyleBase()
        # 文章副标题样式
        self.paper_subheading_style = PaperSubTitleStyleBase()
        # 文章概要
        self.abstract = AbstractStyleBase()
        # 一级标题:段落标题
        self.title_level_1_style = FirstTitleStyleBase()
        # 二级标题:
        self.title_level_2_style = SecondTitleStyleBase()
        # 三级标题
        self.title_level_3_style = ThirdTitleStyleBase()
        # 四级标题
        self.title_level_4_style = FourthTitleStyleBase()
        # 正文样式
        self.content_style = ContentStyleBase()
        # 表格样式
        self.table_title_style = TableTitleStyleBase()  # 标题
        self.table_head_style = TableHeadStyleBase()  # 表头
        self.table_body_style = TableBodyStyleBase()  # 表体
        # 图片样式
        self.picture_title_style = PicTitleStyleBase()

    def generate(self):
        """生成"""
        print('=' * 100)
        print('自动化报告开始: {}'.format(self.filename))
        # 文章段落结构排序
        self._sort_chapter()
        print('文章段落结排序结束!')
        # 页面设置
        self._handle_page_settings()
        print('页面设置初始化成功!')
        # 目录
        # python脚本无法直接实现, 模板文件事先准备好目录部分, 完成后打开文件直接刷新.
        # 文章标题
        self.handle_paper_heading()
        print('页面标题写入成功!')
        # 文章副标题
        self.handle_paper_subheading()
        print('页面副标题写入成功!')
        # 文章摘要
        self.handle_abstract()
        print('文章摘要写入成功!')
        # 文章段落内容
        self._handle_chapter()
        print('文章段落内容处理开始!')
        # 页眉和页脚
        # 模板文件事先准备好
        # 保存
        self.doc_obj.save(self.filename)
        print('报告保存本地成功!')
        # 删除临时文件
        self._clear()
        print('清理缓存文件结束!')
        print('输出文件: {}'.format(self.filename))

    # ===============
    # = 子类需重写方法 =
    # ===============
    def handle_paper_heading(self):
        """文章标题"""
        pass

    def handle_paper_subheading(self):
        """文章副标题"""
        pass

    def handle_abstract(self):
        """文章摘要"""
        pass

    # ===============
    # = 子类可调用方法 =
    # ===============
    @staticmethod
    def delete_paragraph(paragraph):
        """删除段落"""
        p = paragraph._element
        p.getparent().remove(p)
        paragraph._p = paragraph._element = None

    def rotate_page_orientation(self, orientation: Orientation = None):
        """旋转页面方向"""
        section = self.doc_obj.add_section()
        if orientation is None:
            # 页面方向缺失, 恢复默认方向
            orientation = self.default_orientation
        self._set_orientation(section, orientation)

    def lines_break(self, lines: int):
        """换行"""
        for line in range(lines):
            self.doc_obj.add_paragraph()

    def page_break(self):
        """换页"""
        paragraph = self.doc_obj.add_paragraph()
        run = paragraph.add_run()
        run.add_break(WD_BREAK.PAGE)

    def clear_file(self, file):
        """"""
        self._clear_files.add(file)

    # =============
    # == 样式设置 ==
    # =============
    def _load_builtin_style(self):
        """加载所有Word内置样式"""
        for style in WORD_BUILTIN_STYLES:
            item = WORD_BUILTIN_STYLES[style]
            try:
                self.doc_obj.styles.add_style(style, item['type'], builtin=True)
            except:
                pass
        # 设置正文首行缩进为0,防止读取模板情况下,新增内容设置无效!
        self.doc_obj.styles['Normal'].paragraph_format.first_line_indent = Cm(0)

    def _handle_page_settings(self):
        """页面设置"""
        for section in self.doc_obj.sections:
            self._set_orientation(section, self.default_orientation)
            section.left_margin = Cm(self.page_conf['left_margin'])
            section.right_margin = Cm(self.page_conf['right_margin'])
            section.top_margin = Cm(self.page_conf['top_margin'])
            section.bottom_margin = Cm(self.page_conf['bottom_margin'])
            section.page_width = Cm(self.page_conf['page_width'])
            section.page_height = Cm(self.page_conf['page_height'])

    def _set_orientation(self, section, orientation: Orientation = None):
        """页面旋转"""
        if isinstance(orientation, (Orientation,)):
            orientation = orientation.value
        section.orientation = orientation
        # 必须同时设置方向、新宽度和高度值,才会生效
        if orientation == WD_ORIENTATION.PORTRAIT:
            section.page_width = Cm(self.page_conf['page_width'])
            section.page_height = Cm(self.page_conf['page_height'])
        else:
            section.page_height = Cm(self.page_conf['page_width'])
            section.page_width = Cm(self.page_conf['page_height'])

    def _handle_header(self):
        """设置页眉"""
        # 模板自行设置
        pass

    def _handle_footer(self):
        """设置页脚"""
        # 模板自行设置
        # footer = self.doc_obj.sections[0].footer
        # self._add_page_number(footer.paragraphs[0].add_run())
        # footer.paragraphs[0].alignment = ALIGNMENT_DICT['center']
        pass

    def _add_page_number(self, run):
        """页脚添加页码"""
        fldChar1 = self._create_element('w:fldChar')
        self._create_attribute(fldChar1, 'w:fldCharType', 'begin')

        instrText = self._create_element('w:instrText')
        self._create_attribute(instrText, 'xml:space', 'preserve')
        instrText.text = "PAGE"

        fldChar2 = self._create_element('w:fldChar')
        self._create_attribute(fldChar2, 'w:fldCharType', 'end')

        run._r.append(fldChar1)
        run._r.append(instrText)
        run._r.append(fldChar2)

    @staticmethod
    def _create_element(name):
        """创建Oxml元素"""
        return OxmlElement(name)

    @staticmethod
    def _create_attribute(element, name, value):
        """添加页码属性"""
        element.set(ns.qn(name), value)

    # =====================
    # == 章节及段落数据处理 ==
    # =====================
    def _sort_chapter(self):
        """"""
        regex_rules = [
            r'(chapter_\d{2})_(paragraph_\d{2})',
            r'(chapter_\d{2}_\d{2})_(paragraph_\d{2})',
            r'(chapter_\d{2}_\d{2}_\d{2})_(paragraph_\d{2})',
            r'(chapter_\d{2}_\d{2}_\d{2}_\d{2})_(paragraph_\d{2})',
        ]
        chapter_list = list()
        subchapter_dict = dict()
        paragraph_dict = dict()
        for attr in dir(self):
            if attr.startswith('chapter'):
                # 处理标题
                if re.match(r'chapter_[\d_]+$', attr):
                    if re.match(r'chapter_\d{2}$', attr):
                        # 一级标题
                        if attr not in chapter_list:
                            chapter_list.append(attr)
                    else:
                        # 非一级标题
                        if not subchapter_dict.get(attr):
                            subchapter_dict[attr] = set()
                        if not paragraph_dict.get(attr):
                            paragraph_dict[attr] = list()
                        self.mount_parent_chapter(subchapter_dict, attr)
                    continue

                # 处理段落
                for regex_rule in regex_rules:
                    result = re.match(regex_rule, attr)
                    if result:
                        chapter = result.group(1)
                        self.mount_parent_chapter(subchapter_dict, chapter)
                        if not paragraph_dict.get(chapter):
                            paragraph_dict[chapter] = list()
                        paragraph_dict[chapter].append(attr)

        # 排序
        chapter_list = sorted(chapter_list)
        for k, v in subchapter_dict.items():
            subchapter_dict[k] = sorted(v)
        self.chapter_list = chapter_list
        self.subchapter_dict = subchapter_dict
        self.paragraph_dict = paragraph_dict

    def _handle_chapter(self):
        """处理章节"""
        for chapter in self.chapter_list:
            handler = getattr(self, chapter)
            try:
                handler()
            except Exception as e:
                raise ValueError('处理章节标题chapter[{}]处理失败: {}'.format(chapter, e))
            self._handle_subchapters(chapter)

    def mount_parent_chapter(self, subchapter_dict, chapter):
        """挂载父类章节"""
        res = chapter.split('_')
        if len(res) > 2:
            parent_chapter = '_'.join(res[:-1])
            if not subchapter_dict.get(parent_chapter):
                subchapter_dict[parent_chapter] = set()
            subchapter_dict[parent_chapter].add(chapter)
            self.mount_parent_chapter(subchapter_dict, parent_chapter)

    def get_paragraph_data(self, function_name: str = None):
        """获取章段落对应数据"""
        if not function_name:
            function_name = sys._getframe(1).f_code.co_name
        data = self.data.get(function_name)
        return data if data else dict()

    def _handle_subchapters(self, chapter: str):
        """处理子章节"""
        subchapters = self.subchapter_dict.get(chapter, list())
        for subchapter in subchapters:
            handler = getattr(self, subchapter)
            try:
                handler()
            except Exception as e:
                raise ValueError('处理章节标题chapter[{}]处理失败: {}'.format(subchapter, e))

    def _handle_paragraphs(self, chapter):
        """段落"""
        paragraphs = self.paragraph_dict.get(chapter, list())
        for paragraph in paragraphs:
            handler = getattr(self, paragraph)
            try:
                handler()
            except Exception as e:
                raise ValueError('处理段落paragraph[{}]处理失败: {}'.format(paragraph, e))

    def _clear(self):
        """清理临时文件"""
        for f in self._clear_files:
            try:
                os.remove(f)
            except Exception as e:
                pass

    def _handle_filename(self):
        """处理文件名"""
        current = datetime.datetime.now().strftime('%Y%m%d%H%M%S%f')
        if self.filename.endswith('.docx'):
            self.filename = '{}_{}.docx'.format(self.filename[:-5], current)
        elif self.filename.endswith('.doc'):
            self.filename = '{}_{}.doc'.format(self.filename[:-5], current)
        else:
            self.filename += '_.docx'



"""
@File  : title.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  : 
"""

# -*- coding: utf-8 -*-
from docx import Document
from docx.document import Document as DocObject

from smartDocx.config import DocStylesBase
from smartDocx.package.paragraph import Paragraph


class Title(Paragraph):
    """标题"""

    def __init__(self, config: DocStylesBase, content_type='TITLE'):
        super(Title, self).__init__(config=config, content_type=content_type)

    def generate(self, doc_obj: DocObject, text: str, *args, **kwargs):
        """生成标题"""
        title = doc_obj.add_heading(level=self.styles['level'])  # 添加新段落
        run_obj = title.add_run(text=text)  # 新段落添加文本
        self._format(title, run_obj)

    def _format(self, title, run_obj):
        """格式化"""
        self._format_paragraph(title)  # 新段落格式化
        self._format_font(run_obj=run_obj)  # 字体格式化



"""
@File  : base.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  : 自动化报告
"""
# -*- coding: utf-8 -*-

from docx.oxml.ns import qn
from docx.shared import Pt, RGBColor

from smartDocx.config import DocStylesBase
from smartDocx.constants import DEFAULT_WEST_FONT


class Base(object):
    """基类"""

    def __init__(self, content_type: str, config: (DocStylesBase, dict), *args, **kwargs):
        """

        :param content_type: 类型
        :param config: 配置
        :param args:
        :param kwargs:
        """
        self.type = content_type
        self.styles = config._to_dict() if isinstance(config, (DocStylesBase,)) else config
        self.args = args
        self.kwargs = kwargs

    def generate(self, *args, **kwargs):
        """生成文本"""
        raise Exception("子类需重写该方法")

    def _format(self, *args, **kwargs):
        """格式化"""
        pass

    # ==============
    # = 字体格式设置 =
    # ==============
    def _handle_font_weight(self, obj, value):
        """设置字体粗细"""
        if value is True:
            obj.bold = True
        else:
            obj.bold = False

    def _handle_font_size(self, obj, value):
        """设置字体大小"""
        if isinstance(value, (int, float)):
            obj.font.size = Pt(value)

    def _handle_color(self, obj, value):
        """设置字体颜色"""
        if isinstance(value, str):
            obj.font.color.rgb = self._handle_rgb_hex_str(rgb_hex_str=value)

    def _handle_font_type(self, obj, value):
        """设置字体类型"""
        if isinstance(value, str):
            # 西文字体
            obj.font.name = self.styles.get('west_font_type', DEFAULT_WEST_FONT)
            # 中文字体
            obj._element.rPr.rFonts.set(*(qn('w:eastAsia'), value))

    def _handle_italic(self, obj, value):
        """设置斜体"""
        if value is True:
            obj.font.italic = True
        else:
            obj.font.italic = False

    def _handle_underline(self, obj, value):
        """设置下划线"""
        if isinstance(value, int):
            obj.underline = value

    def _handle_rgb_hex_str(self, rgb_hex_str):
        """处理十六进制RGB值"""
        r = int(rgb_hex_str[1:3], 16)
        g = int(rgb_hex_str[3:5], 16)
        b = int(rgb_hex_str[5:], 16)
        return RGBColor(r, g, b)



"""
@File  : table.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/15
@Desc  : 
"""

# -*- coding: utf-8 -*-
from docx.document import Document as DocObject
from docx.oxml import OxmlElement, parse_xml
from docx.oxml.ns import qn, nsdecls
from docx.shared import Cm
from docx.table import Table as DocTable, _Cell

from smartDocx.config import DocStylesBase
from smartDocx.constants import LENGTH_SWIFT, TableVerticalAlignment, TableLandscapeAlignment
from smartDocx.package.base import Base
from smartDocx.package.content import Content


class Table(Base):
    """表格"""

    def __init__(self, config: DocStylesBase, content_type: str = 'TABLE', *args, **kwargs):
        super(Table, self).__init__(content_type=content_type, config=config, *args, **kwargs)
        self.body_styles = self.styles
        if kwargs.get('head_config'):
            head_config = kwargs['head_config']
            self.head_styles = head_config._to_dict() if isinstance(head_config, DocStylesBase) else head_config
        else:
            self.head_styles = self.body_styles
        self.column_width_dict = None
        self.table_head_width_dict = None
        self.table = None
        self.columns = None
        self.data = None
        self.max_merge_col_index = None
        self.doc_obj = None

    def generate(self, doc_obj: DocObject, columns: list, data: list, table_title: str = None,
                 table_title_style: DocStylesBase = None, built_table_color_style: str = '',
                 max_merge_col_index: int = None, horizontal_merge=True, vertical_merge=True):
        """
        生成表格
        :param doc_obj:
        :param columns: 表头列表
        :param data: 标题数据
        :param table_title: 表格标题
        :param table_title_style: 表格标题样式
        :param built_table_color_style: 内置表格样式
        :param max_merge_col_index: 合并单元格作用最大
        :param horizontal_merge: 水平方向合并单元格
        :param vertical_merge: 垂直方向合并单元格
        :return:
        """
        self.doc_obj = doc_obj
        self.columns = columns
        self.data = data
        self.max_merge_col_index = max_merge_col_index
        self.table_head_width_dict = {str(index): self._calculate_character_length(value) for index, value in
                                      enumerate(columns)}
        # 1.生成表标题
        if table_title:
            content = Content(config=table_title_style)
            content.generate(doc_obj, text=table_title)
        # 2.生成表格主体
        col_len = len(columns)
        row_len = len(data) + 1
        self.table = doc_obj.add_table(row_len, col_len)  # type: DocTable
        if built_table_color_style:
            self.table.style = built_table_color_style
        # 3.数据填充
        row_index = 0
        self.column_width_dict = {str(index): 0 for index in range(len(columns))}
        row_index = self._create_table_head(columns, row_index)
        self._fill_cell_data(data, row_index, horizontal_merge, vertical_merge)
        # 4.表格样式格式化
        self._format_table()
        self._justify_column_width()

    def _create_table_head(self, columns: list, row_index: int):
        """生成表头"""
        self.head_styles['font_weight'] = True
        self._fill_row_data(row=columns, row_index=row_index, is_table_head=True)
        return row_index + 1

    def _fill_cell_data(self, data: list, row_index: int, horizontal_merge: bool, vertical_merge: bool):
        """填充数据"""
        self.body_styles['font_weight'] = False
        for row in data:
            self._fill_row_data(row=row, row_index=row_index, horizontal_merge=horizontal_merge)
            # 垂直方向合并单元格
            if vertical_merge and self.max_merge_col_index is not None:
                self._merge_vertical_cells(row_index=row_index)
            row_index += 1

    def _fill_row_data(self, row: list, row_index: int, is_table_head: bool = False, horizontal_merge: bool = True):
        """填充行数据"""
        for col_index, value in enumerate(row):
            cell = self.table.cell(row_index, col_index)
            # 1.单元格样式设置
            self._format_cell_border(cell)
            run_obj = cell.paragraphs[0].add_run(str(value))
            # 2.单元格文字设置
            self._format_font(cell, run_obj, is_table_head)
            # 3.记录字符串长度
            value_len = self._calculate_character_length(str(value))
            if value_len > self.column_width_dict[str(col_index)]:
                self.column_width_dict[str(col_index)] = value_len
            # 4.水平方向合并单元格
            if horizontal_merge and self.max_merge_col_index is not None:
                self._merge_horizontal_cells(row_index=row_index, col_index=col_index)

        # 5.设置行高
        height = self.styles['font_size'] * LENGTH_SWIFT['pt'] * 2
        self.table.rows[row_index].height = Cm(height)

    def _merge_vertical_cells(self, row_index: int):
        """纵向合并单元格"""
        data_row_index = row_index - 1
        if row_index > 1:
            for col_index in range(0, self.max_merge_col_index + 1):
                if col_index > 0:
                    if self.data[data_row_index - 1][col_index - 1] != self.data[data_row_index][col_index - 1]:
                        continue
                pre = self.data[data_row_index - 1][col_index]
                cur = self.data[data_row_index][col_index]
                if cur and (pre == cur):
                    cell_ = self.table.cell(row_index - 1, col_index)  # 垂直方向上一个单元格
                    cell_text_ = cell_.paragraphs[0].text
                    cell = self.table.cell(row_index, col_index)
                    cell_text = cell.paragraphs[0].text
                    cell.paragraphs[0].text = ''  # 删除被合并单元格数据
                    try:
                        cell_.merge(cell)
                    except Exception as e:
                        # 合并单元格失败,恢复原始文本
                        cell_.paragraphs[0].text = cell_text_
                        cell.paragraphs[0].text = cell_text
                    else:
                        cell_.text = cell_.text.strip()

    def _merge_horizontal_cells(self, row_index: int, col_index: int):
        """横向合并单元格"""
        data_row_index = row_index - 1
        if 0 <= col_index <= self.max_merge_col_index:
            pre = self.data[data_row_index][col_index - 1]
            cur = self.data[data_row_index][col_index]
            if cur and (pre == cur):
                cell_ = self.table.cell(row_index, col_index - 1)  # 水平方向前一个单元格
                cell_text_ = cell_.paragraphs[0].text
                cell = self.table.cell(row_index, col_index)
                cell_text = cell.paragraphs[0].text
                cell.paragraphs[0].text = ''  # 删除被合并单元格数据
                try:
                    cell_.merge(cell)
                except Exception as e:
                    # 合并单元格失败,恢复原始文本
                    cell_.paragraphs[0].text = cell_text_
                    cell.paragraphs[0].text = cell_text
                else:
                    cell_.text = cell_.text.strip()

    def _format_table(self):
        """表格对齐"""
        for cell in self.table._cells:
            cell.paragraphs[0].paragraph_format.alignment = self.styles['alignment'] if self.styles[
                'alignment'] else TableLandscapeAlignment.center
            cell.paragraphs[0].paragraph_format.space_before = 0  # 段前设置
            cell.paragraphs[0].paragraph_format.space_after = 0  # 段后设置
            cell.paragraphs[0].paragraph_format.first_line_indent = Cm(0)  # 首航缩进为0
            cell.vertical_alignment = self.styles['vertical_alignment'] if self.styles[
                'vertical_alignment'] else TableVerticalAlignment.center
        self.table.alignment = self.styles['alignment']

    def _format_font(self, cell, run_obj, is_table_head: bool = False):
        """格式化文字"""
        font_styles = [
            'font_size',
            'font_weight',
            'color',
            'font_type',
            'italic',
            'underline',
            'background'
        ]
        if is_table_head:
            styles = self.head_styles  # 表头样式
        else:
            styles = self.body_styles  # 表体样式
        for font_style in font_styles:
            value = styles.get(font_style, None)
            handle_func = getattr(self, '_handle_{}'.format(font_style))
            if handle_func:
                if font_style == 'background':
                    handle_func(cell, value)
                else:
                    handle_func(run_obj, value)

    def _handle_background(self, cell, value):
        """单元格背景色设置"""
        if value:
            shading_elm_1 = parse_xml(r'<w:shd {} w:fill="{}"/>'.format(nsdecls('w'), value))
            cell._tc.get_or_add_tcPr().append(shading_elm_1)

    @staticmethod
    def _format_cell_border(cell: _Cell, **kwargs):
        """
        设置单元格边框函数
        使用方法:
        Set_cell_border(
            cell,
            top={"sz": 12, "val": "single", "color": "#FF0000", "space": "0"},
            bottom={"sz": 12, "color": "#00FF00", "val": "single"},
            start={"sz": 24, "val": "dashed", "shadow": "true"},
            end={"sz": 12, "val": "dashed"},
        )
        传入参数有cell, 即单元格；top指上边框；bottom指下边框；start指左边框；end指右边框。
        "sz"指线的粗细程度；"val"指线型，比如单线，虚线等；"color"指颜色，颜色编码可百度；
        "space"指间隔，一般不设置，设置的值大于0会导致线错开；"shadow"指边框阴影
        """
        default_style = {"sz": 5, "color": "#000000", "val": "single"}
        if not kwargs.get('top'):
            kwargs['top'] = default_style
        if not kwargs.get('bottom'):
            kwargs['bottom'] = default_style
        if not kwargs.get('start'):
            kwargs['start'] = default_style
        if not kwargs.get('end'):
            kwargs['end'] = default_style

        tc = cell._tc
        tcPr = tc.get_or_add_tcPr()

        tcBorders = tcPr.first_child_found_in("w:tcBorders")
        if tcBorders is None:
            tcBorders = OxmlElement('w:tcBorders')
            tcPr.append(tcBorders)

        for edge in ('start', 'top', 'end', 'bottom', 'insideH', 'insideV'):
            edge_data = kwargs.get(edge)
            if edge_data:
                tag = 'w:{}'.format(edge)

                element = tcBorders.find(qn(tag))
                if element is None:
                    element = OxmlElement(tag)
                    tcBorders.append(element)

                for key in ["sz", "val", "color", "space", "shadow"]:
                    if key in edge_data:
                        element.set(qn('w:{}'.format(key)), str(edge_data[key]))

    def _justify_column_width(self):
        """表格宽度自适应 会重新设置字体样式"""
        self.table.autofit = False  # 关闭自带的自适应
        self._calculate_appropriate_column_width()  # 计算合适的列宽度
        for row_index in range(len(self.table.rows)):
            for col_index in range(len(self.columns)):
                cell = self.table.cell(row_index, col_index)
                width = self.column_width_dict[str(col_index)]
                cell.width = Cm(width)
                if row_index == 0:
                    self._format_font(cell=cell, run_obj=cell.paragraphs[0].runs[0], is_table_head=True)
                else:
                    self._format_font(cell=cell, run_obj=cell.paragraphs[0].runs[0])

    def _calculate_character_length(self, character):
        """
        计算字符串长度
        :param character:
        :return:
        """
        character_len = len(str(character))
        unicode_len = len(str(character).encode('utf-8'))
        if character_len == unicode_len:
            # ASCII码字符组成
            # Word文本字符长度
            appearance_len = round(character_len * self.styles['font_size'] * LENGTH_SWIFT['pt'] / 2, 2)
        else:
            # 非ASCII和ASCII字符混合
            # 汉字Unicode编码占3个字节,ASCII码占1个字节
            ascii_len = (unicode_len - character_len) / 2
            noascii_len = (unicode_len - ascii_len) / 3
            # Word文本字符长度
            appearance_len = round(
                character_len * self.styles['font_size'] * LENGTH_SWIFT['pt'] / 2 + noascii_len * self.styles[
                    'font_size'] * LENGTH_SWIFT['pt'], 2)
        return round(appearance_len * 1.3, 2)  # 考虑到表格单元格内边距

    def _calculate_appropriate_column_width(self):
        """计算合适的单元格宽度"""
        page_width_cm = self.doc_obj.sections[-1].page_width.cm
        page_left_cm = self.doc_obj.sections[-1].left_margin.cm
        page_right_cm = self.doc_obj.sections[-1].right_margin.cm
        TABLE_TOTAL_WIDTH = page_width_cm - page_right_cm - page_left_cm  # 表格宽度
        col_width_sum = sum(self.column_width_dict.values())
        col_num = len(self.column_width_dict)
        col_width_avg = round(col_width_sum / col_num, 3)
        redundant_len = TABLE_TOTAL_WIDTH - col_width_sum
        if redundant_len < 0:
            col_great_then_avg = {k: v for k, v in self.column_width_dict.items() if v > col_width_avg}
            col_less_and_equal_then_avg = {k: v for k, v in self.column_width_dict.items() if
                                           v <= col_width_avg}
            sum_great_then_avg = sum(col_great_then_avg.values())
            sum_less_and_equal_then_avg = sum(col_less_and_equal_then_avg.values())
            redundant_len_ = TABLE_TOTAL_WIDTH - sum_less_and_equal_then_avg
            if redundant_len_ >= 0:
                for k, v in col_great_then_avg.items():
                    self.column_width_dict[k] = (v / sum_great_then_avg * redundant_len_)
            else:
                for k, v in self.column_width_dict.items():
                    self.column_width_dict[k] = (v / col_width_sum * TABLE_TOTAL_WIDTH)
        else:
            for k, v in self.column_width_dict.items():
                self.column_width_dict[k] = (v / col_width_sum * redundant_len) + v



"""
@File  : paragraph.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  : 
"""

# -*- coding: utf-8 -*-
import re

from docx import Document
from docx.document import Document as DocObject
from docx.oxml import OxmlElement
from docx.oxml.ns import qn
from docx.shared import Pt, Inches

from smartDocx.config import DocStylesBase
from smartDocx.package.base import Base


class Paragraph(Base):
    """段落"""

    def __init__(self, config: DocStylesBase, content_type: str = 'PARAGRAPH', *args, **kwargs):
        super(Paragraph, self).__init__(config=config, content_type=content_type, *args, **kwargs)
        self._run_obj = None

    def generate(self, doc_obj: DocObject, text: str, append=False):
        """生成段落"""
        if append:
            # 去除上一个run_obj的换行符号
            paragraph = doc_obj.paragraphs[-1]
            prev_run_obj = paragraph.runs[-1]
            prev_run_obj.text = re.sub('\t', '', prev_run_obj.text)
            prev_run_obj.text = re.sub('\n', '', prev_run_obj.text)
        else:
            paragraph = doc_obj.add_paragraph()  # 添加新段落
        self._run_obj = paragraph.add_run(text=text)  # 新段落添加文本
        self._format(paragraph, self._run_obj)  # 字体格式化

    def _format(self, paragraph, run_obj):
        """格式化"""
        # 新段落格式化
        self._format_paragraph(paragraph)
        # 字体格式化
        self._format_font(run_obj=run_obj)

    def _format_paragraph(self, paragraph):
        """段落样式设置"""
        paragraph_styles = [
            'text_align',
            'line_indent',
            'line_spacing',
            'space_before',
            'space_after',
            'paragraph_underline'
        ]
        for paragraph_style in paragraph_styles:
            value = self.styles.get(paragraph_style, None)
            handle_func = getattr(self, '_handle_{}'.format(paragraph_style))
            if handle_func:
                handle_func(paragraph, value)

    def _format_font(self, run_obj):
        """字体样式设置"""
        font_styles = [
            'font_size',
            'font_weight',
            'color',
            'font_type',
            'italic',
            'underline'
        ]
        for font_style in font_styles:
            value = self.styles.get(font_style, None)
            handle_func = getattr(self, '_handle_{}'.format(font_style))
            if handle_func:
                handle_func(run_obj, value)

    # ==============
    # = 段落格式设置 =
    # ==============
    def _handle_line_indent(self, obj, value):
        """设置首航缩进"""
        if value is True:
            obj.paragraph_format.first_line_indent = Inches(0.4)
        else:
            obj.paragraph_format.first_line_indent = Inches(0)

    def _handle_line_spacing(self, obj, value):
        """设置行间距"""
        if isinstance(value, (int, float)):
            obj.paragraph_format.line_spacing = value

    def _handle_space_before(self, obj, value):
        """设置段前"""
        if isinstance(value, (int, float)):
            obj.paragraph_format.space_before = Pt(value)

    def _handle_space_after(self, obj, value):
        """设置段后"""
        if isinstance(value, (int, float)):
            obj.paragraph_format.space_after = Pt(value)

    def _handle_text_align(self, obj, value):
        """设置对齐方式"""
        obj.alignment = value

    def _handle_paragraph_underline(self, paragraph, value):
        """段落底部下划线设置"""
        if value is None:
            return

        prPr = paragraph._element.get_or_add_pPr()
        kwargs = dict()
        kwargs['bottom'] = value

        tcBorders = prPr.first_child_found_in("w:tcBorders")
        if tcBorders is None:
            tcBorders = OxmlElement('w:tcBorders')
            prPr.append(tcBorders)

        for edge in ('start', 'top', 'end', 'bottom', 'insideH', 'insideV'):
            edge_data = kwargs.get(edge)
            if edge_data:
                tag = 'w:{}'.format(edge)

                element = tcBorders.find(qn(tag))
                if element is None:
                    element = OxmlElement(tag)
                    tcBorders.append(element)

                for key in ["sz", "val", "color", "space", "shadow"]:
                    if key in edge_data:
                        element.set(qn('w:{}'.format(key)), str(edge_data[key]))



"""
@File  : content.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/16
@Desc  : 
"""
# -*- coding: utf-8 -*-
from smartDocx.config import DocStylesBase
from smartDocx.package.paragraph import Paragraph


class Content(Paragraph):
    """正文内容"""

    def __init__(self, config: DocStylesBase, content_type='CONTENT'):
        super(Content, self).__init__(config=config, content_type=content_type)



"""
@File  : picture.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/15
@Desc  : 
"""

# -*- coding: utf-8 -*-
from docx.document import Document as DocObject
from docx.shared import Cm

from smartDocx.config import DocStylesBase, PicImgStyleBase
from smartDocx.package.content import Content


class Picture(object):
    """图片"""

    def __init__(self, config: PicImgStyleBase):
        self.config = config
        if not (0 < self.config.zom_rate <= 1):
            self.width = 15
        else:
            self.width = round(15 * self.config.zom_rate, 3)

    def generate(self, doc_obj: DocObject, pic_path, pic_title: str = None,
                 pic_title_style: DocStylesBase = None):
        """
        生成图片
        :param doc_obj:
        :param pic_path: 图片地址
        :param pic_title: 图片标题
        :param pic_title_style: 图片标题样式
        :return:
        """
        if pic_title and pic_title_style:
            content = Content(config=pic_title_style)
            content.generate(doc_obj=doc_obj, text=pic_title)
        paragraph = doc_obj.add_paragraph()
        paragraph.alignment = self.config.alignment.value
        run = paragraph.add_run()
        run.add_picture(image_path_or_stream=pic_path, width=Cm(self.width))



# """
# @File  : test.py
# @IDE   : PyCharm
# @Author: Sanbom
# @Date  : 2021/7/13
# @Desc  :
# """
# # -*- coding: utf-8 -*-
#
# from kits.smart_report.package.content import Content
# from kits.smart_report.package.paragraph import Paragraph
# from kits.smart_report.package.table import Table
# from kits.smart_report.package.title import Title
# from kits.smart_report.report import SmartReportDataBase, SmartReportBase
# from kits.smart_report.utils import put_paragraph_data, handle_paragraph
#
#
# class SmartReportData(SmartReportDataBase):
#     """智能报告数据"""
#
#     def __init__(self, sql_client=None, neo4j_client=None, es_client=None, *args,
#                  **kwargs):
#         super(SmartReportData, self).__init__(sql_client, neo4j_client, es_client, *args,
#                                               **kwargs)
#
#     @put_paragraph_data
#     def chapter_01_paragraph_01(self):
#         """章节01"""
#         return dict(a='{}_a'.format('chapter_01'))
#
#     @put_paragraph_data
#     def chapter_01_01_paragraph_01(self):
#         """章节1.1"""
#         data = dict(
#             a='{}_a'.format('chapter_01_0a=1'),
#             columns=['市级责任单位', '工单批次', '事项批次'],
#             data=[
#                 ['上海市房屋管理局', 796, 613],
#                 ['上海市交通委员会', 294, 239],
#                 ['上海市住房和城乡建设管理委员会', 96, 85],
#                 ['上海市城市管', 67, 54],
#                 ['上海市绿化和市容管理局(上海市林业局)', 42, 38],
#                 ['上海市公积金管理中心', 19, 9],
#                 ['上海市水务局（上海市海洋局）', 7, 6],
#             ],
#             table_title='表xxx 表格标题'
#         )
#         return data
#
#     @put_paragraph_data
#     def chapter_01_01_01_paragraph_01(self):
#         """章节1.1.1"""
#         data = dict(
#             a='{}_a'.format('chapter_01_01_01'),
#             chart_01=dict(
#                 xaxis=["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"],
#                 yaxis=[
#                     [5, 20, 36, 10, 75, 90],
#                     [15, 6, 45, 20, 35, 66]
#                 ],
#                 legend=['商家A', '商家B']
#             )
#         )
#         return data
#
#     @put_paragraph_data
#     def chapter_01_01_01_01_paragraph_01(self):
#         """章节1.1.1.1"""
#         return dict(a='{}_a'.format('chapter_01_01_01_01'))
#
#     @put_paragraph_data
#     def chapter_01_02_paragraph_01(self):
#         """章节1.2"""
#         data = dict(
#             columns=['责任单位', '信访事项', '批次'],
#             data=[
#                 ['崇明区', '城乡建设 - 住房保障与房地产 - 房地产开发管理 ', 174],
#                 ['浦东新区', '城乡建设 - 住房保障与房地产 - 物业服务', 84],
#                 ['黄浦区', '城乡建设 - 国有土地上房屋征收与补偿 - 安置补偿', 63],
#                 ['浦东新区', '城乡建设 - 住房保障与房地产 - 房地产开发管理', 52],
#             ],
#             table_title='表xxx 表格标题'
#         )
#         return data
#
#     @put_paragraph_data
#     def chapter_02_paragraph_01(self):
#         """章节2"""
#         return dict(a='{}_a'.format('chapter_02'))
#
#     @put_paragraph_data
#     def chapter_03_paragraph_01(self):
#         """章节3"""
#         return dict(a='{}_a'.format('chapter_03'))
#
#
# class SmartReport(SmartReportBase):
#     """智能报告"""
#
#     def __init__(self, filename: str, data: dict, *args, template: str = None, **kwargs):
#         super(SmartReport, self).__init__(filename, data, *args, template=template, **kwargs)
#
#     def handle_paper_heading(self):
#         """文章标题"""
#         title = Content(config=self.title_level_0_style)
#         title.generate(doc_obj=self.doc_obj, text='自动化测试报告')
#
#     def handle_paper_subheading(self):
#         """文章副标题"""
#         paragraph = Paragraph(config=self.paper_subheading_style)
#         paragraph.generate(doc_obj=self.doc_obj, text='时间范围: 2021年1月7日 --- 2021年2月7日')
#
#     def handle_abstract(self):
#         """文章摘要"""
#         content = Content(config=self.abstract)
#         text = "[本分析报告所指的信访数据以国家信访系统上海分系统登记的来信、来访、网信为依据，涵盖建设交通系统" \
#                "市级党政机关、委属事业单位、代管的中央在沪单位。市住建委、市交通委、市水务局、市绿化市容局、市房管" \
#                "局、市城管执法局、市公积金中心为上海市信访办管理系统中独立承办的责任单位。市建设交通党委代管的中央在" \
#                "沪单位由市住建委代为转交，由具体单位承办。]"
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01(self):
#         """章节1"""
#         p_title = Title(config=self.title_level_1_style)
#         text = "一、总体情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01(self):
#         """章节1.1 标题"""
#         p_title = Title(config=self.title_level_2_style)
#         text = "1.1 情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01_paragraph_01(self):
#         """章节1.1 段落01"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "新华社北京7月13日电 中国共产党成立100周年庆祝活动总结会议13日上午在京举行。中共中央总书记、国家主席、" \
#                "中央军委主席习近平在人民大会堂亲切会见庆祝活动筹办工作各方面代表，向他们表示衷心感谢和诚挚问候，对他们的辛" \
#                "勤工作和优异成绩给予充分肯定，勉励大家奋发进取、再创佳绩。{a}".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#         table = Table(config=self.table_body_style)
#         table.generate(doc_obj=self.doc_obj, columns=chapter_data.get('columns'), data=chapter_data.get('data'),
#                        table_title=chapter_data.get('table_title'), table_title_style=self.table_title_style)
#
#     @handle_paragraph
#     def chapter_01_01_01(self):
#         """章节1.1.1 标题"""
#         p_title = Title(config=self.title_level_3_style)
#         text = "1.1.1 总体情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01_01_paragraph_01(self):
#         """章节1.1.1 段落1"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "新华社北京7月13日电 中国共产党成立100周年庆祝活动总结会议13日上午在京举行。中共中央总书记、国家主席、" \
#                "中央军委主席习近平在人民大会堂亲切会见庆祝活动筹办工作各方面代表，向他们表示衷心感谢和诚挚问候，对他们的辛" \
#                "勤工作和优异成绩给予充分肯定，勉励大家奋发进取、再创佳绩。{a}".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01_01_01(self):
#         """章节1.1.1.1"""
#         p_title = Title(config=self.title_level_4_style)
#         text = "1.1.1.1 四级标题"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01_01_01_paragraph_01(self):
#         """章节1.1.1.1 段落01"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "四级标题测试内容。{a}".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_02(self):
#         """章节01.02 标题"""
#         p_title = Title(config=self.title_level_2_style)
#         text = "1.2 总体情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_02_paragraph_01(self):
#         """章节01.02 段落1"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "新华社北京7月13日电 中国共产党成立100周年庆祝活动总结会议13日上午在京举行。中共中央总书记、国家主席、" \
#                "中央军委主席习近平在人民大会堂亲切会见庆祝活动筹办工作各方面代表，向他们表示衷心感谢和诚挚问候，对他们的辛" \
#                "勤工作和优异成绩给予充分肯定，勉励大家奋发进取、再创佳绩。".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#         chapter_data['columns'] = ['t1', 't2', 't3', 't4', 't5', 't6']
#         chapter_data['data'] = [
#             ['测试', 'c1', 'c2', 0, 0, 0],
#             ['测试', 'c1', 'c2', 0, 0, 0],
#             ['测试', 'c1', 'c3', 0, 0, 0],
#             ['测试1', 'c1', 'c2', 0, 0, 0],
#             ['测试1', 'c3', 'c2', 0, 0, 0],
#         ]
#         table = Table(config=self.table_body_style)
#         table.generate(doc_obj=self.doc_obj, columns=chapter_data.get('columns'), data=chapter_data.get('data'),
#                        table_title=chapter_data.get('table_title'), table_title_style=self.table_title_style,
#                        built_table_color_style='Medium Shading 1 Accent 1', max_merge_col_index=2)
#         self.page_break()
#
#     @handle_paragraph
#     def chapter_02(self):
#         """章节02 标题"""
#         p_title = Title(config=self.title_level_1_style)
#         text = "二、情况1"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_02_paragraph_01(self):
#         """章节02 段落01"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "中共中央政治局常委李克强、汪洋、赵乐际、韩正参加会见。中共中央政治局常委王沪宁参加会见并在总结会议上讲话。{a}".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_03(self):
#         """章节03 标题"""
#         p_title = Title(config=self.title_level_1_style)
#         text = "三、情况2"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_03_paragraph_01(self):
#         """章节03 段落1"""
#         chapter_data = self.get_paragraph_data()
#         content = Content(config=self.content_style)
#         text = "13日上午，人民大会堂北大厅灯光璀璨、气氛热烈。11时许，习近平等来到这里，全场响起热烈掌声" \
#                "。习近平等走到代表们中间，同大家亲切交流并合影留念。{a}".format(**chapter_data)
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#
# if __name__ == '__main__':
#     # 从文件创建文档对象
#     file = '/Users/sanbom/Desktop/需求调查样例V0.7.1.docx'
#     # # template =None
#     # template = '/Users/sanbom/Storage/MyCode/kits/smart_report/template/demo.docx'
#     template = '/Users/sanbom/Storage/MyCode/kits/smart_report/template/需求调查.docx'
#     # # template = '/Users/sanbom/Storage/MyCode/kits/smart_report/template/table_styles.docx'
#     # template = '/Users/sanbom/Storage/MyCode/kits/smart_report/template/xxx.docx'
#     data = SmartReportData().data
#     print(data)
#     report = SmartReport(filename=file, data=data, template=template)
#     report.generate()
#
#
# from kits.smart_report import SmartReportBase, Content, Paragraph, handle_paragraph, Title
#
#
# class SmartReport(SmartReportBase):
#     """智能报告"""
#
#     def __init__(self, filename: str, data: dict, *args, template: str = None, **kwargs):
#         super(SmartReport, self).__init__(filename, data, *args, template=template, **kwargs)
#
#     def handle_paper_heading(self):
#         """文章标题"""
#         title = Content(config=self.title_level_0_style)
#         title.generate(doc_obj=self.doc_obj, text='自动化测试报告')
#
#     def handle_paper_subheading(self):
#         """文章副标题"""
#         paragraph = Paragraph(config=self.paper_subheading_style)
#         paragraph.generate(doc_obj=self.doc_obj, text='时间范围: 2021年1月7日 --- 2021年2月7日')
#
#     def handle_abstract(self):
#         """文章摘要"""
#         content = Content(config=self.abstract)
#         text = "[本分析报告所指的信访数据以国家信访系统上海分系统登记的来信、来访、网信为依据，涵盖建设交通系统" \
#                "市级党政机关、委属事业单位、代管的中央在沪单位。市住建委、市交通委、市水务局、市绿化市容局、市房管" \
#                "局、市城管执法局、市公积金中心为上海市信访办管理系统中独立承办的责任单位。市建设交通党委代管的中央在" \
#                "沪单位由市住建委代为转交，由具体单位承办。]"
#         content.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01(self):
#         """章节1"""
#         p_title = Title(config=self.title_level_1_style)
#         text = "一、总体情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)
#
#     @handle_paragraph
#     def chapter_01_01(self):
#         """章节1.1 标题"""
#         p_title = Title(config=self.title_level_2_style)
#         text = "1.1 情况"
#         p_title.generate(doc_obj=self.doc_obj, text=text)

from kits.smart_report import SmartReportBase, Content
from kits.smart_report.config import ContentStyleBase

report = SmartReportBase(filename='test.docx', data=dict())
doc_obj = report.doc_obj
content = Content(config=ContentStyleBase())
content.generate(doc_obj, text="自动化报告,从空白文件创建!")
report.generate()



"""
@File  : constants.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/23
@Desc  : 
"""
# -*- coding: utf-8 -*-
import os
from enum import Enum

from docx.enum.section import WD_ORIENTATION
from docx.enum.style import WD_STYLE_TYPE
from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT, WD_TABLE_ALIGNMENT
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT

# 根目录
BASE_DIR = os.path.dirname(os.path.abspath(__file__)).replace('\\', '/')
TMP_DIR = os.path.join(BASE_DIR, 'tmp').replace('\\', '/')
TEMPLATE_DIR = os.path.join(BASE_DIR, 'template').replace('\\', '/')
DATA_DIR = os.path.join(BASE_DIR, 'data').replace('\\', '/')


class DocFontSize(Enum):
    """文档字体磅值"""
    Zero = 36  # 初号
    SmallZero = 31  # 小初号
    One = 27  # 一号
    SmallOne = 24  # 小一号
    Two = 21
    SmallTwo = 18
    Three = 16
    SmallThree = 15
    Four = 14
    SmallFour = 12
    Five = 10.5
    SmallFive = 9
    Six = 8
    SmallSix = 7
    Seven = 6
    SmallSeven = 5


# 长度单位转换
LENGTH_SWIFT = {
    'inches': 2.54,  # 英寸
    'cm': 1,  # 厘米
    'mm': 0.1,  # 毫米
    'pt': 0.0376,  # 点或磅
    'twips': 1 / 567000,  # 缇
}


class DocFontFamily(Enum):
    """中文字体格式"""
    ST = '宋体'
    HT = '黑体'
    MSYH = '微软雅黑'
    KT = '楷体'


class Orientation(Enum):
    """页面排布方向"""
    # 纸张方向: 默认纵向(PORTRAIT), 横向(LANDSCAPE)
    landscape = WD_ORIENTATION.LANDSCAPE
    portrait = WD_ORIENTATION.PORTRAIT


class DocAlignment(Enum):
    """文本水平对齐方式"""
    left = WD_PARAGRAPH_ALIGNMENT.LEFT
    right = WD_PARAGRAPH_ALIGNMENT.RIGHT
    center = WD_PARAGRAPH_ALIGNMENT.CENTER
    justify = WD_PARAGRAPH_ALIGNMENT.JUSTIFY


class TableLandscapeAlignment(Enum):
    """表格单元格文本, 水平对齐方式"""
    left = WD_TABLE_ALIGNMENT.LEFT
    right = WD_TABLE_ALIGNMENT.RIGHT
    center = WD_TABLE_ALIGNMENT.CENTER


class TableVerticalAlignment(Enum):
    """表格单元格文本,垂直对齐方式"""
    top = WD_CELL_VERTICAL_ALIGNMENT.TOP
    bottom = WD_CELL_VERTICAL_ALIGNMENT.BOTTOM
    center = WD_CELL_VERTICAL_ALIGNMENT.CENTER


# 默认表格样式
class WordBuiltinStyle(object):
    """
    word默认表格背景色样式
        样式对应效果,请查看template文件夹下的table_styles.docx文件

        #### 生成代码如下
        import docx

        from smart_smart_report.config import WordTableBackgroundStyle
        document = docx.Document()
        for style_ in document.styles:
            name = style_.name
            if name in WordTableBackgroundStyle()._data:
                document.add_paragraph(text=name)
                table = document.add_table(7, 3, style=style_)
                document.add_paragraph()
        document.save('xxx.docx')

    """

    def __init__(self):
        self._data = {
            "Normal": {'type_name': 'paragraph', 'name': 'Normal', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Header": {'type_name': 'paragraph', 'name': 'Header', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Footer": {'type_name': 'paragraph', 'name': 'Footer', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Heading 1": {'type_name': 'paragraph', 'name': 'Heading 1', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 2": {'type_name': 'paragraph', 'name': 'Heading 2', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 3": {'type_name': 'paragraph', 'name': 'Heading 3', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 4": {'type_name': 'paragraph', 'name': 'Heading 4', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 5": {'type_name': 'paragraph', 'name': 'Heading 5', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 6": {'type_name': 'paragraph', 'name': 'Heading 6', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 7": {'type_name': 'paragraph', 'name': 'Heading 7', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 8": {'type_name': 'paragraph', 'name': 'Heading 8', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Heading 9": {'type_name': 'paragraph', 'name': 'Heading 9', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "No Spacing": {'type_name': 'paragraph', 'name': 'No Spacing', 'type': WD_STYLE_TYPE.PARAGRAPH,
                           'zh_name': ''},
            "Title": {'type_name': 'paragraph', 'name': 'Title', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Subtitle": {'type_name': 'paragraph', 'name': 'Subtitle', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "List Paragraph": {'type_name': 'paragraph', 'name': 'List Paragraph', 'type': WD_STYLE_TYPE.PARAGRAPH,
                               'zh_name': ''},
            "Body Text": {'type_name': 'paragraph', 'name': 'Body Text', 'type': WD_STYLE_TYPE.PARAGRAPH,
                          'zh_name': ''},
            "Body Text 2": {'type_name': 'paragraph', 'name': 'Body Text 2', 'type': WD_STYLE_TYPE.PARAGRAPH,
                            'zh_name': ''},
            "Body Text 3": {'type_name': 'paragraph', 'name': 'Body Text 3', 'type': WD_STYLE_TYPE.PARAGRAPH,
                            'zh_name': ''},
            "List": {'type_name': 'paragraph', 'name': 'List', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "List 2": {'type_name': 'paragraph', 'name': 'List 2', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "List 3": {'type_name': 'paragraph', 'name': 'List 3', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "List Bullet": {'type_name': 'paragraph', 'name': 'List Bullet', 'type': WD_STYLE_TYPE.PARAGRAPH,
                            'zh_name': ''},
            "List Bullet 2": {'type_name': 'paragraph', 'name': 'List Bullet 2', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "List Bullet 3": {'type_name': 'paragraph', 'name': 'List Bullet 3', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "List Number": {'type_name': 'paragraph', 'name': 'List Number', 'type': WD_STYLE_TYPE.PARAGRAPH,
                            'zh_name': ''},
            "List Number 2": {'type_name': 'paragraph', 'name': 'List Number 2', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "List Number 3": {'type_name': 'paragraph', 'name': 'List Number 3', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "List Continue": {'type_name': 'paragraph', 'name': 'List Continue', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "List Continue 2": {'type_name': 'paragraph', 'name': 'List Continue 2', 'type': WD_STYLE_TYPE.PARAGRAPH,
                                'zh_name': ''},
            "List Continue 3": {'type_name': 'paragraph', 'name': 'List Continue 3', 'type': WD_STYLE_TYPE.PARAGRAPH,
                                'zh_name': ''},
            "macro": {'type_name': 'paragraph', 'name': 'macro', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Quote": {'type_name': 'paragraph', 'name': 'Quote', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Caption": {'type_name': 'paragraph', 'name': 'Caption', 'type': WD_STYLE_TYPE.PARAGRAPH, 'zh_name': ''},
            "Intense Quote": {'type_name': 'paragraph', 'name': 'Intense Quote', 'type': WD_STYLE_TYPE.PARAGRAPH,
                              'zh_name': ''},
            "TOC Heading": {'type_name': 'paragraph', 'name': 'TOC Heading', 'type': WD_STYLE_TYPE.PARAGRAPH,
                            'zh_name': ''},
            "Header Char": {'type_name': 'character', 'name': 'Header Char', 'type': WD_STYLE_TYPE.CHARACTER,
                            'zh_name': ''},
            "Footer Char": {'type_name': 'character', 'name': 'Footer Char', 'type': WD_STYLE_TYPE.CHARACTER,
                            'zh_name': ''},
            "Default Paragraph Font": {'type_name': 'character', 'name': 'Default Paragraph Font',
                                       'CHARACTER': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Heading 1 Char": {'type_name': 'character', 'name': 'Heading 1 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 2 Char": {'type_name': 'character', 'name': 'Heading 2 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 3 Char": {'type_name': 'character', 'name': 'Heading 3 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Title Char": {'type_name': 'character', 'name': 'Title Char', 'type': WD_STYLE_TYPE.CHARACTER,
                           'zh_name': ''},
            "Subtitle Char": {'type_name': 'character', 'name': 'Subtitle Char', 'type': WD_STYLE_TYPE.CHARACTER,
                              'zh_name': ''},
            "Body Text Char": {'type_name': 'character', 'name': 'Body Text Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Body Text 2 Char": {'type_name': 'character', 'name': 'Body Text 2 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                                 'zh_name': ''},
            "Body Text 3 Char": {'type_name': 'character', 'name': 'Body Text 3 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                                 'zh_name': ''},
            "Macro Text Char": {'type_name': 'character', 'name': 'Macro Text Char', 'type': WD_STYLE_TYPE.CHARACTER,
                                'zh_name': ''},
            "Quote Char": {'type_name': 'character', 'name': 'Quote Char', 'type': WD_STYLE_TYPE.CHARACTER,
                           'zh_name': ''},
            "Heading 4 Char": {'type_name': 'character', 'name': 'Heading 4 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 5 Char": {'type_name': 'character', 'name': 'Heading 5 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 6 Char": {'type_name': 'character', 'name': 'Heading 6 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 7 Char": {'type_name': 'character', 'name': 'Heading 7 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 8 Char": {'type_name': 'character', 'name': 'Heading 8 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Heading 9 Char": {'type_name': 'character', 'name': 'Heading 9 Char', 'type': WD_STYLE_TYPE.CHARACTER,
                               'zh_name': ''},
            "Strong": {'type_name': 'character', 'name': 'Strong', 'type': WD_STYLE_TYPE.CHARACTER, 'zh_name': ''},
            "Emphasis": {'type_name': 'character', 'name': 'Emphasis', 'type': WD_STYLE_TYPE.CHARACTER, 'zh_name': ''},
            "Intense Quote Char": {'type_name': 'character', 'name': 'Intense Quote Char',
                                   'type': WD_STYLE_TYPE.CHARACTER, 'zh_name': ''},
            "Subtle Emphasis": {'type_name': 'character', 'name': 'Subtle Emphasis', 'type': WD_STYLE_TYPE.CHARACTER,
                                'zh_name': ''},
            "Intense Emphasis": {'type_name': 'character', 'name': 'Intense Emphasis', 'type': WD_STYLE_TYPE.CHARACTER,
                                 'zh_name': ''},
            "Subtle Reference": {'type_name': 'character', 'name': 'Subtle Reference', 'type': WD_STYLE_TYPE.CHARACTER,
                                 'zh_name': ''},
            "Intense Reference": {'type_name': 'character', 'name': 'Intense Reference',
                                  'type': WD_STYLE_TYPE.CHARACTER, 'zh_name': ''},
            "Book Title": {'type_name': 'character', 'name': 'Book Title', 'type': WD_STYLE_TYPE.CHARACTER,
                           'zh_name': ''},
            "Normal Table": {'type_name': 'table', 'name': 'Normal Table', 'type': WD_STYLE_TYPE.TABLE, 'zh_name': ''},
            "Table Grid": {'type_name': 'table', 'name': 'Table Grid', 'type': WD_STYLE_TYPE.TABLE, 'zh_name': ''},
            "Light Shading": {'type_name': 'table', 'name': 'Light Shading', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Light Shading Accent 1": {'type_name': 'table', 'name': 'Light Shading Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light Shading Accent 2": {'type_name': 'table', 'name': 'Light Shading Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light Shading Accent 3": {'type_name': 'table', 'name': 'Light Shading Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light Shading Accent 4": {'type_name': 'table', 'name': 'Light Shading Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light Shading Accent 5": {'type_name': 'table', 'name': 'Light Shading Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light Shading Accent 6": {'type_name': 'table', 'name': 'Light Shading Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Light List": {'type_name': 'table', 'name': 'Light List', 'type': WD_STYLE_TYPE.TABLE, 'zh_name': ''},
            "Light List Accent 1": {'type_name': 'table', 'name': 'Light List Accent 1', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light List Accent 2": {'type_name': 'table', 'name': 'Light List Accent 2', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light List Accent 3": {'type_name': 'table', 'name': 'Light List Accent 3', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light List Accent 4": {'type_name': 'table', 'name': 'Light List Accent 4', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light List Accent 5": {'type_name': 'table', 'name': 'Light List Accent 5', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light List Accent 6": {'type_name': 'table', 'name': 'Light List Accent 6', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid": {'type_name': 'table', 'name': 'Light Grid', 'type': WD_STYLE_TYPE.TABLE, 'zh_name': ''},
            "Light Grid Accent 1": {'type_name': 'table', 'name': 'Light Grid Accent 1', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid Accent 2": {'type_name': 'table', 'name': 'Light Grid Accent 2', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid Accent 3": {'type_name': 'table', 'name': 'Light Grid Accent 3', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid Accent 4": {'type_name': 'table', 'name': 'Light Grid Accent 4', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid Accent 5": {'type_name': 'table', 'name': 'Light Grid Accent 5', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Light Grid Accent 6": {'type_name': 'table', 'name': 'Light Grid Accent 6', 'type': WD_STYLE_TYPE.TABLE,
                                    'zh_name': ''},
            "Medium Shading 1": {'type_name': 'table', 'name': 'Medium Shading 1', 'type': WD_STYLE_TYPE.TABLE,
                                 'zh_name': ''},
            "Medium Shading 1 Accent 1": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 1',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 1 Accent 2": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 2',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 1 Accent 3": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 3',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 1 Accent 4": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 4',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 1 Accent 5": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 5',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 1 Accent 6": {'type_name': 'table', 'name': 'Medium Shading 1 Accent 6',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2": {'type_name': 'table', 'name': 'Medium Shading 2', 'type': WD_STYLE_TYPE.TABLE,
                                 'zh_name': ''},
            "Medium Shading 2 Accent 1": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 1',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2 Accent 2": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 2',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2 Accent 3": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 3',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2 Accent 4": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 4',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2 Accent 5": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 5',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium Shading 2 Accent 6": {'type_name': 'table', 'name': 'Medium Shading 2 Accent 6',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Medium List 1": {'type_name': 'table', 'name': 'Medium List 1', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Medium List 1 Accent 1": {'type_name': 'table', 'name': 'Medium List 1 Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 1 Accent 2": {'type_name': 'table', 'name': 'Medium List 1 Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 1 Accent 3": {'type_name': 'table', 'name': 'Medium List 1 Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 1 Accent 4": {'type_name': 'table', 'name': 'Medium List 1 Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 1 Accent 5": {'type_name': 'table', 'name': 'Medium List 1 Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 1 Accent 6": {'type_name': 'table', 'name': 'Medium List 1 Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2": {'type_name': 'table', 'name': 'Medium List 2', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Medium List 2 Accent 1": {'type_name': 'table', 'name': 'Medium List 2 Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2 Accent 2": {'type_name': 'table', 'name': 'Medium List 2 Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2 Accent 3": {'type_name': 'table', 'name': 'Medium List 2 Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2 Accent 4": {'type_name': 'table', 'name': 'Medium List 2 Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2 Accent 5": {'type_name': 'table', 'name': 'Medium List 2 Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium List 2 Accent 6": {'type_name': 'table', 'name': 'Medium List 2 Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1": {'type_name': 'table', 'name': 'Medium Grid 1', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Medium Grid 1 Accent 1": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1 Accent 2": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1 Accent 3": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1 Accent 4": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1 Accent 5": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 1 Accent 6": {'type_name': 'table', 'name': 'Medium Grid 1 Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2": {'type_name': 'table', 'name': 'Medium Grid 2', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Medium Grid 2 Accent 1": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2 Accent 2": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2 Accent 3": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2 Accent 4": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2 Accent 5": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 2 Accent 6": {'type_name': 'table', 'name': 'Medium Grid 2 Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3": {'type_name': 'table', 'name': 'Medium Grid 3', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Medium Grid 3 Accent 1": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3 Accent 2": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3 Accent 3": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3 Accent 4": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3 Accent 5": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Medium Grid 3 Accent 6": {'type_name': 'table', 'name': 'Medium Grid 3 Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Dark List": {'type_name': 'table', 'name': 'Dark List', 'type': WD_STYLE_TYPE.TABLE, 'zh_name': ''},
            "Dark List Accent 1": {'type_name': 'table', 'name': 'Dark List Accent 1', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Dark List Accent 2": {'type_name': 'table', 'name': 'Dark List Accent 2', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Dark List Accent 3": {'type_name': 'table', 'name': 'Dark List Accent 3', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Dark List Accent 4": {'type_name': 'table', 'name': 'Dark List Accent 4', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Dark List Accent 5": {'type_name': 'table', 'name': 'Dark List Accent 5', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Dark List Accent 6": {'type_name': 'table', 'name': 'Dark List Accent 6', 'type': WD_STYLE_TYPE.TABLE,
                                   'zh_name': ''},
            "Colorful Shading": {'type_name': 'table', 'name': 'Colorful Shading', 'type': WD_STYLE_TYPE.TABLE,
                                 'zh_name': ''},
            "Colorful Shading Accent 1": {'type_name': 'table', 'name': 'Colorful Shading Accent 1',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful Shading Accent 2": {'type_name': 'table', 'name': 'Colorful Shading Accent 2',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful Shading Accent 3": {'type_name': 'table', 'name': 'Colorful Shading Accent 3',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful Shading Accent 4": {'type_name': 'table', 'name': 'Colorful Shading Accent 4',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful Shading Accent 5": {'type_name': 'table', 'name': 'Colorful Shading Accent 5',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful Shading Accent 6": {'type_name': 'table', 'name': 'Colorful Shading Accent 6',
                                          'type': WD_STYLE_TYPE.TABLE,
                                          'zh_name': ''},
            "Colorful List": {'type_name': 'table', 'name': 'Colorful List', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Colorful List Accent 1": {'type_name': 'table', 'name': 'Colorful List Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful List Accent 2": {'type_name': 'table', 'name': 'Colorful List Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful List Accent 3": {'type_name': 'table', 'name': 'Colorful List Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful List Accent 4": {'type_name': 'table', 'name': 'Colorful List Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful List Accent 5": {'type_name': 'table', 'name': 'Colorful List Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful List Accent 6": {'type_name': 'table', 'name': 'Colorful List Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid": {'type_name': 'table', 'name': 'Colorful Grid', 'type': WD_STYLE_TYPE.TABLE,
                              'zh_name': ''},
            "Colorful Grid Accent 1": {'type_name': 'table', 'name': 'Colorful Grid Accent 1',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid Accent 2": {'type_name': 'table', 'name': 'Colorful Grid Accent 2',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid Accent 3": {'type_name': 'table', 'name': 'Colorful Grid Accent 3',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid Accent 4": {'type_name': 'table', 'name': 'Colorful Grid Accent 4',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid Accent 5": {'type_name': 'table', 'name': 'Colorful Grid Accent 5',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "Colorful Grid Accent 6": {'type_name': 'table', 'name': 'Colorful Grid Accent 6',
                                       'type': WD_STYLE_TYPE.TABLE,
                                       'zh_name': ''},
            "No List": {'type_name': 'numbering', 'name': 'No List', 'type': WD_STYLE_TYPE.LIST, 'zh_name': ''}
        }

    @property
    def data(self):
        return self._data

    def __contains__(self, item):
        if item in self._data:
            return True
        return False

    def __getitem__(self, item):
        if item in self:
            return self._data[item]
        raise KeyError('无该类别样式: '.format(item))

    def __iter__(self):
        for item in self.data:
            yield item


WORD_BUILTIN_STYLES = WordBuiltinStyle()

# 默认英文字体
DEFAULT_WEST_FONT = 'Times New Roman'

if __name__ == '__main__':
    center = DocAlignment.center
    print(center.value)



"""
@File  : echart.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/16
@Desc  : 
"""
# -*- coding: utf-8 -*-
import datetime
import os

from pyecharts.faker import _Faker
from pyecharts.render import make_snapshot
from snapshot_phantomjs import snapshot

from smartDocx.constants import TMP_DIR


class Chart(object):
    """
    echarts图表
        该类仅实现生成图片功能(该类的render方法),其余配置请参考官方文档。


        # 参考文档及链接
        1.pyecharts
            官网文档: https://pyecharts.org/#/zh-cn/intro
            官方示例: https://gallery.pyecharts.org/#/README
        2.echarts
            官方文档: https://echarts.apache.org/zh/feature.html
            官方示例: https://echarts.apache.org/examples/zh/index.html

    """

    def __init__(self, tmp_dir: str = None):
        """
        :param tmp_dir:
        """
        if tmp_dir:
            if os.path.exists(tmp_dir):
                raise FileNotFoundError
        else:
            self.tmp_dir = TMP_DIR  # 临时文件夹
        self.picture_type = 'png'  # 图片类型
        self.pic_path = None  # 图片绝对路径
        self.chart_obj = None  # 原始pyecharts的图对象
        self._faker = _Faker

    def demo(self):
        """样例"""
        # self.chart_obj = Bar()

    @staticmethod
    def get_map_max_value(array, count: int = 5):
        """
        获取地区图例最大值
        @param array: 数组
        @param count: 均分数量
        """
        max_value = max(array)
        if max_value <= 10:
            return 10
        avg = max_value // count
        while avg * count * 0.8 <= max_value:
            if avg % 5 == 0 and (max_value <= avg * count):
                break
            avg += 1
        return avg * count

    def render(self, del_html=True):
        """
        渲染html,获取图片路径
            需要提前安装好node,实现html文件的截图;
        :param del_html: 是否删除html文件,默认删除,防止垃圾文件占用磁盘空间
        :return:
        """
        if not self.chart_obj:
            raise ValueError('chart_obj为None')
        html_path = self._get_path(prefix='html', file_type='html')
        pic_path = self._get_path(prefix='pic', file_type=self.picture_type)
        self.chart_obj.render(html_path)
        make_snapshot(snapshot, html_path, pic_path)
        if del_html:
            self.remove(html_path)
        return pic_path

    def _get_path(self, prefix, file_type):
        """
        获取路径
            临时文件夹tmp中存放pyecharts渲染生成的html和图片文件;
        :param prefix: 前缀
        :param file_type: 文件类型
        :return:
        """
        params = {
            'current': datetime.datetime.now().strftime('%Y%m%d%H%M%S%f'),
            'prefix': prefix,
            'file_type': file_type
        }
        picture_path = os.path.join(self.tmp_dir, '{prefix}_{current}.{file_type}'.format(**params))
        return picture_path.replace('\\', '/')

    @staticmethod
    def remove(file_path):
        """
        删除文件
            html转图片后,删除原html文件
        :param file_path: 文件绝对路径
        :return:
        """
        try:
            os.remove(file_path)
        except Exception as e:
            print(e)



"""
@File  : __init__.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  : 
"""
# -*- coding: utf-8 -*-

__version__ = "0.0.4"

__all__ = (
    'Paragraph',
    'Title',
    'Table',
    'Picture',
    'Chart',
    'Content',
    'SmartReportBase',
    'SmartReportDataBase',
    'put_paragraph_data',
    'handle_paragraph',
    'utils'
)

from .report import SmartReportBase, SmartReportDataBase
from .package.chart import Chart
from .package.content import Content
from .package.paragraph import Paragraph
from .package.picture import Picture
from .package.table import Table
from .package.title import Title
from .utils import put_paragraph_data, handle_paragraph



"""
@File  : setup.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/10/3
@Desc  : 
"""
# -*- coding: utf-8 -*-

# encoding: utf-8

from setuptools import setup, find_packages

setup(
    name="smartDocx",  # egg包名
    version="0.0.4",
    description="smart docx",
    author="sanbom",
    author_email="sanbomsu1993@outlook.com",
    zip_safe=False,
    license='MIT',
    # 依赖包
    install_requires=[
        "xlrd>=1.2.0, <2",
        "numpy>=1.21.2",
        "openpyxl>=3.0.9",
        "pandas>=1.3.3",
        "pyecharts==1.9.0",
        "pyecharts-snapshot==0.2.0",
        "snapshot-phantomjs==0.0.3",
        "python-docx>=0.8.11"
    ],
    # 包含src中所有的包,并排除一些特定的包
    packages=find_packages("src",
                           exclude=["*.tests", "*.tests.*"]),
    package_dir={"": "src"},  # 告诉setuptools在src目录下找包
    package_data={
        "": ["*.txt"],  # 任何包中含有txt的文件
        "smartDocx": ["template/*.docx", "README.md"],  # demo包中data目录下的dat文件
    }
)



"""
@File  : utils.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/14
@Desc  : 
"""

# -*- coding: utf-8 -*-
import functools
import inspect
import time
from typing import Any

from smartDocx import SmartReportBase


def get_function_name():
    """获取正在运行函数(或方法)名称"""
    return inspect.stack()[1][3]


def put_paragraph_data(message):
    """将段落数据归类"""

    def decorator(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            """嵌套函数"""
            frame = inspect.currentframe().f_back
            # 获取被装饰方法的实例对象
            self_obj = frame.f_locals['self']  # SmartReportDataBase类型
            # 获取被装饰方法的函数名称
            function_name = func.__name__
            # 返回章节数据
            chapter_data = func(*args, **kwargs)
            # 获取章节层级
            chapter_level = self_obj.get_chapter_level(function_name)
            # 绑定该章节层级数据
            self_obj._data[chapter_level] = chapter_data

        return inner

    if callable(message):
        func = message
        return decorator(func)
    else:
        return decorator


def handle_paragraph(message):
    """处理段落"""

    def decorator(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            """嵌套函数"""
            frame = inspect.currentframe().f_back
            # 1.获取被装饰方法的实例对象
            self_obj = frame.f_locals['self']  # type:SmartReportBase
            # 2.获取被装饰方法的函数名称
            function_name = func.__name__
            # 3.执行函数前操作
            pass
            # 4.执行章节处理函数
            func(*args, **kwargs)
            # 5.函数执行后执行
            time.sleep(0.1)
            # 6.处理段落
            self_obj._handle_paragraphs(chapter=function_name)

        return inner

    if callable(message):
        func = message
        return decorator(func)
    else:
        return decorator


def config_dict_to_obj(data: dict):
    """字典转对象属性"""

    def handle_dict(obj: object, dict_data: dict):
        """处理字典数据"""
        for k, v in dict_data.items():
            obj_ = get_obj(v)
            setattr(obj, k, obj_)

    def get_obj(value: Any):
        """设置属性"""
        if isinstance(value, dict):
            obj_ = type('', (object,), dict())
            for k, v in value.items():
                setattr(obj_, k, get_obj(v))
            return obj_
        else:
            return value

    config_obj = type('', (object,), dict())
    try:
        handle_dict(obj=config_obj, dict_data=data)
    except Exception as e:
        pass
    finally:
        return config_obj


def print_func_params(output: bool = False):
    """打印函数的参数"""

    def decorator(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            """嵌套函数"""
            # 获取被装饰方法的函数名称
            function_name = func.__name__
            if output:
                print("=" * 10 + "  output_func_params  " + "=" * 10)
                print("func_name: {}".format(function_name))
                print("args: {}".format(args))
                print("kwargs: {}".format(kwargs))
                print("=" * 10 + "  output_func_params  " + "=" * 10)
            # 执行
            func(*args, **kwargs)

        return inner

    if callable(output):
        func = output
        return decorator(func)
    else:
        return decorator



"""
@File  : constants.py
@IDE   : PyCharm
@Author: Sanbom
@Date  : 2021/7/13
@Desc  :
"""
# -*- coding: utf-8 -*-

from smartDocx.constants import DocFontSize, DocFontFamily, DocAlignment, TableLandscapeAlignment, \
    TableVerticalAlignment, Orientation


# =========================
# =====  默认文本样式   =====
# =========================
class DocStylesBase(object):
    """文档样式基类"""

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def _to_dict(self):
        """转字典"""
        styles = dict()
        for attr in dir(self):
            if attr.startswith('_'):
                continue
            value = getattr(self, attr)
            # 自定义的Enum类型,需转成实际数值
            if isinstance(value, (DocFontFamily, DocAlignment, TableLandscapeAlignment,
                                  TableVerticalAlignment, DocFontSize, Orientation)):
                value = value.value
            styles[attr] = value
        return styles

    def __iter__(self):
        for attr in self._to_dict():
            yield attr

    def __getitem__(self, item):
        return self._to_dict()[item]


class PageStyleBase(DocStylesBase):
    """页面样式基类"""

    def __init__(self,
                 page_width: (float, int) = 21.00,
                 page_height: (float, int) = 29.70,
                 left_margin: (int, float) = 3.17,
                 right_margin: (int, float) = 3.17,
                 top_margin: (int, float) = 2.54,
                 bottom_margin: (int, float) = 2.54,
                 orientation: Orientation = Orientation.portrait,
                 **kwargs):
        """
        :param page_width: 页面宽度
        :param page_height: 页面高度
        :param left_margin: 左边距cm
        :param right_margin: 右边距cm
        :param top_margin: 上边距cm
        :param bottom_margin: 底边距cm
        :param orientation: 纸张方向: 默认纵向portrait
        :param kwargs:
        """
        super(PageStyleBase, self).__init__(
            page_width=page_width,
            page_height=page_height,
            left_margin=left_margin,
            right_margin=right_margin,
            top_margin=top_margin,
            bottom_margin=bottom_margin,
            orientation=orientation,
            **kwargs)


class ParagraphStyleBase(DocStylesBase):
    """段落样式基类"""

    def __init__(self,
                 font_size=DocFontSize.SmallFour,
                 text_align=DocAlignment.justify,
                 font_weight=False,
                 color='#000000',
                 font_type=DocFontFamily.MSYH,
                 west_font_type=DocFontFamily.MSYH,
                 line_indent=True,
                 line_spacing=None,
                 style='Normal',
                 space_before=5,
                 space_after=5,
                 italic=False,
                 **kwargs):
        super(ParagraphStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            font_weight=font_weight,
            color=color,
            font_type=font_type,
            west_font_type=west_font_type,
            line_indent=line_indent,
            line_spacing=line_spacing,
            style=style,
            space_before=space_before,
            space_after=space_after,
            italic=italic,
            **kwargs)


class TitleStyleBase(DocStylesBase):
    """标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Four,
                 text_align=DocAlignment.justify,
                 font_weight=False,
                 color='#000000',
                 font_type=DocFontFamily.MSYH,
                 west_font_type=DocFontFamily.MSYH,
                 level=1,
                 line_indent=False,
                 line_spacing=None,
                 style='Normal',
                 space_before=5,
                 space_after=5,
                 italic=False,
                 underline=0,
                 **kwargs):
        """
        :param font_size: 字体大小
        :param text_align: 对齐模式
        :param font_weight: 字体粗细
        :param color: 字体颜色
        :param font_type: 中文字体类型
        :param west_font_type: 西文字体类型
        :param level: 标题等级
        :param line_indent: 首航缩进
        :param line_spacing: 行间距
        :param style: 字体类型
        :param space_before: 段前间距
        :param space_after: 段后间距
        :param italic: 是否斜体
        :param underline: 下划线
        :param kwargs:
        """
        super(TitleStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            font_weight=font_weight,
            color=color,
            font_type=font_type,
            west_font_type=west_font_type,
            level=level,
            line_indent=line_indent,
            line_spacing=line_spacing,
            style=style,
            space_before=space_before,
            space_after=space_after,
            italic=italic,
            underline=underline,
            **kwargs)


# 文章标题
class PaperTitleStyleBase(TitleStyleBase):
    """文章标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.SmallTwo,
                 text_align=DocAlignment.center,
                 font_weight=True,
                 level=0,
                 line_spacing=1.5,
                 space_before=10,
                 **kwargs):
        super(PaperTitleStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


class PaperSubTitleStyleBase(TitleStyleBase):
    """文章副标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.SmallFour,
                 text_align=DocAlignment.center,
                 font_weight=True,
                 level=0,
                 line_spacing=1.5,
                 space_before=10,
                 **kwargs):
        super(PaperSubTitleStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


# 正文
class ContentStyleBase(DocStylesBase):
    """正文样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Four,
                 text_align=DocAlignment.justify,
                 font_weight=False,
                 color='#000000',
                 font_type=DocFontFamily.MSYH,
                 west_font_type=DocFontFamily.MSYH,
                 line_indent=True,
                 line_spacing=1.5,
                 style='Normal',
                 space_before=5,
                 space_after=5,
                 italic=False,
                 paragraph_underline=None,
                 **kwargs):
        super(ContentStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            font_weight=font_weight,
            color=color,
            font_type=font_type,
            west_font_type=west_font_type,
            line_indent=line_indent,
            line_spacing=line_spacing,
            style=style,
            space_before=space_before,
            space_after=space_after,
            italic=italic,
            paragraph_underline=paragraph_underline,
            **kwargs)


class AbstractStyleBase(ContentStyleBase):
    """摘要样式基类"""

    def __init__(self,
                 font_size=DocFontSize.SmallFour,
                 font_weight=True,
                 font_type=DocFontFamily.KT,
                 west_font_type=DocFontFamily.MSYH,
                 line_spacing=1,
                 space_before=10,
                 space_after=30,
                 **kwargs):
        super(AbstractStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            font_type=font_type,
            west_font_type=west_font_type,
            line_spacing=line_spacing,
            space_before=space_before,
            space_after=space_after,
            **kwargs)


# 段落标题
class FirstTitleStyleBase(TitleStyleBase):
    """段落: 一级标题样式基类"""

    def __init__(self,
                 font_weight=True,
                 level=1,
                 line_spacing=1.5,
                 space_before=10,
                 **kwargs):
        super(FirstTitleStyleBase, self).__init__(
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


class SecondTitleStyleBase(TitleStyleBase):
    """段落: 二级级标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Four,
                 font_weight=True,
                 level=2,
                 line_spacing=1.5,
                 space_before=5,
                 **kwargs):
        super(SecondTitleStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


class ThirdTitleStyleBase(TitleStyleBase):
    """段落: 三级标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.SmallFour,
                 font_weight=True,
                 level=3,
                 line_spacing=1.5,
                 space_before=10,
                 **kwargs):
        super(ThirdTitleStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


class FourthTitleStyleBase(TitleStyleBase):
    """段落: 四级标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Five,
                 font_weight=True,
                 level=4,
                 line_spacing=1.5,
                 space_before=10,
                 **kwargs):
        super(FourthTitleStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            **kwargs)


class UnderlineTitleStyleBase(TitleStyleBase):
    """下划线标题(正文内容)"""

    def __init__(self,
                 font_size=DocFontSize.Five,
                 text_align=DocAlignment.justify,
                 color='#042D86',
                 font_weight=True,
                 line_indent=False,
                 line_spacing=1,
                 level=None,
                 space_before=10,
                 space_after=5,
                 paragraph_underline=None,
                 **kwargs):
        """
        :param font_size:
        :param text_align:
        :param color:
        :param font_weight:
        :param line_indent:
        :param line_spacing:
        :param level:
        :param space_before:
        :param space_after:
        :param paragraph_underline: 段落下划线样式
        :param kwargs:
        """
        if paragraph_underline is None:
            paragraph_underline = {
                "sz": 15,  # 粗细程度
                "color": "#042D86",  # 颜色
                "val": "single"  # 线条类型
            }
        super(UnderlineTitleStyleBase, self).__init__(
            font_size=font_size,
            text_align=text_align,
            color=color,
            font_weight=font_weight,
            line_indent=line_indent,
            line_spacing=line_spacing,
            level=level,
            space_before=space_before,
            space_after=space_after,
            paragraph_underline=paragraph_underline,
            **kwargs)


# 表格
class TableStyleBase(DocStylesBase):
    """表格样式基类"""

    def __init__(self,
                 alignment=TableLandscapeAlignment.center,
                 vertical_alignment=TableVerticalAlignment.center,
                 font_size=DocFontSize.Five,
                 font_type=DocFontFamily.MSYH,
                 west_font_type=DocFontFamily.MSYH,
                 font_weight=False,
                 color='#000000',
                 italic=False,
                 underline=0,
                 autofit=True,
                 background=None,
                 **kwargs):
        """

        :param alignment: 水平对齐方式
        :param vertical_alignment: 垂直对齐方式
        :param font_size:
        :param font_type:
        :param west_font_type:
        :param font_weight:
        :param color:
        :param italic:
        :param underline:
        :param autofit: 自动单元格宽度
        :param background: 单元格背景色
        :param kwargs:
        """
        super(TableStyleBase, self).__init__(
            alignment=alignment,
            vertical_alignment=vertical_alignment,
            font_size=font_size,
            font_type=font_type,
            west_font_type=west_font_type,
            font_weight=font_weight,
            color=color,
            italic=italic,
            underline=underline,
            autofit=autofit,
            background=background,
            **kwargs)


class TableTitleStyleBase(TitleStyleBase):
    """表格标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Five,
                 font_weight=True,
                 level=None,
                 line_spacing=1.5,
                 space_before=5,
                 space_after=0,
                 underline=0,
                 background=None,
                 **kwargs):
        super(TableTitleStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            space_after=space_after,
            underline=underline,
            background=background,
            **kwargs)


class TableHeadStyleBase(TitleStyleBase):
    """表格首部样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Five,
                 font_weight=True,
                 level=None,
                 line_spacing=1.5,
                 space_before=5,
                 space_after=0,
                 background=None,
                 **kwargs):
        super(TableHeadStyleBase, self).__init__(
            font_size=font_size,
            font_weight=font_weight,
            level=level,
            line_spacing=line_spacing,
            space_before=space_before,
            space_after=space_after,
            background=background,
            **kwargs)


class TableBodyStyleBase(TableStyleBase):
    """表格内容样式基类"""

    def __init__(self, **kwargs):
        super(TableBodyStyleBase, self).__init__(**kwargs)


# 图片
class PicTitleStyleBase(TitleStyleBase):
    """图片标题样式基类"""

    def __init__(self,
                 font_size=DocFontSize.Five,
                 level=None,
                 **kwargs):
        super(PicTitleStyleBase, self).__init__(
            font_size=font_size,
            level=level,
            **kwargs)


class PicImgStyleBase(DocStylesBase):
    """图片样式基类"""

    def __init__(self,
                 zom_rate=1,
                 alignment=DocAlignment.center,
                 **kwargs):
        """

        :param zom_rate: 缩放比例
        :param alignment: 对齐方式
        :param kwargs:
        """
        super(PicImgStyleBase, self).__init__(
            zom_rate=zom_rate,
            alignment=alignment,
            **kwargs)
